//<script>
//****************************************************************
//  2000 Microsoft Corporation. All rights reserved.
//****************************************************************
//
// JScript Listbox Object
//
// This object allows for a windows media player skin to "simulate"
// a listbox using buttons, text controls, and subviews.
//****************************************************************



//****************************************************************
//****************************************************************
//
// listbox object
//
//****************************************************************
//****************************************************************

function listbox( lb_sv, items_text, selected_text, selected_sv, focus_button, scrollup_button, scrolldown_button, scroll_button, scrollbacking_button,
                  onSelectionChange )
{
    // Controls
    
    this.lb_sv                  = lb_sv;
    this.items_text             = items_text;
    this.selected_text          = selected_text;
    this.selected_sv            = selected_sv;
    this.focus_button           = focus_button;
    this.scrollup_button        = scrollup_button;
    this.scrolldown_button      = scrolldown_button;
    this.scroll_button          = scroll_button;
    this.scrollbacking_button   = scrollbacking_button;

    // Internal variables
    
    this.fDragging              = false;
    this.nYOffset               = 0;
    this.nCount                 = 0;
    this.nPixelsPerLine         = 0;
    this.nSelectedIndex         = 0;
    this.fScrollingUp           = false;
    this.fScrollingUpHover      = false;
    this.fScrollingDown         = false;
    this.fScrollingDownHover    = false;
    this.fScrollingBack         = false;
    this.fScrollingBackHove     = false;
    this.fScrollingBackDir      = 1;
    
    // Public methods
    
    this.setupLB                        = setupLB;
    this.onTextMouseDownLB              = onTextMouseDownLB;
    this.onHeightChangeLB               = onHeightChangeLB;
    this.onBeginDragLB                  = onBeginDragLB;
    this.onDragScrollbarLB              = onDragScrollbarLB;
    this.onKeyDownLB                    = onKeyDownLB;
    this.onScrollUpMouseDownLB          = onScrollUpMouseDownLB;
    this.onScrollUpEndMoveLB            = onScrollUpEndMoveLB;
    this.onScrollUpMouseMoveLB          = onScrollUpMouseMoveLB;
    this.onScrollUpMouseUpLB            = onScrollUpMouseUpLB;
    this.onScrollDownMouseDownLB        = onScrollDownMouseDownLB;
    this.onScrollDownEndMoveLB          = onScrollDownEndMoveLB;
    this.onScrollDownMouseMoveLB        = onScrollDownMouseMoveLB;
    this.onScrollDownMouseUpLB          = onScrollDownMouseUpLB;
    this.onScrollBackMouseDownLB        = onScrollBackMouseDownLB;
    this.onScrollBackEndMoveLB          = onScrollBackEndMoveLB;
    this.onScrollBackMouseMoveLB        = onScrollBackMouseMoveLB;
    this.onScrollBackMouseUpLB          = onScrollBackMouseUpLB;
    
    // Public Events
    
    this.onSelectionChange              = onSelectionChange;
    
    // Internal Methods
    
    this.setSelectedIndexLB             = setSelectedIndexLB;
    this.adjustScrollbarForSelectionLB  = adjustScrollbarForSelectionLB;
    this.scrollListboxLB                = scrollListboxLB;
    this.adjustSelectedItemLB           = adjustSelectedItemLB;
    this.adjustScrollbarPositionLB      = adjustScrollbarPositionLB;
    this.scrollTextLB                   = scrollTextLB;

    // Initialization Code

    this.scrollup_button.tabstop        = false;
    this.scrolldown_button.tabstop      = false;
    this.scroll_button.tabstop          = false;
    this.scrollbacking_button.tabstop   = false;
}


//****************************************************************
// This function is called to setup the listbox for displaying
// information

function setupLB( text, nItems )
{
    // Give the text to the two text controls
    
    this.items_text.value               = text;
    this.selected_text.value            = text;
    
    this.nCount                         = nItems;
    this.nPixelsPerLine                 = (nItems == 0 ? 0 : (this.items_text.height / nItems));
    
    this.setSelectedIndexLB( 0 );

    this.adjustSelectedItemLB();
    this.onHeightChangeLB();
}



//****************************************************************
// XML Callbacks
//****************************************************************

//****************************************************************
// We get this call when the user has clicked on the text control
// holding our items

function onTextMouseDownLB()
{
    if (event.button == 1)
    {
        var nItem   = Math.floor(event.offsetY / this.nPixelsPerLine);
        
        if (nItem >= 0 && nItem < this.nCount)
        {
            view.focusObjectID          = this.focus_button.id;
            this.setSelectedIndexLB( nItem );
            this.adjustSelectedItemLB();
        }
    }
}

//****************************************************************
// This function is called when the height of the listbox changes

function onHeightChangeLB()
{
    this.adjustScrollbarPositionLB( this.scroll_button.top );
}

//****************************************************************
// This function is called when the mouse goes down on the scrollbar

function onBeginDragLB()
{
    if (event.button == 1 && !this.fDragging)
    {
        this.fDragging  = true;
        this.nYOffset   = event.offsetY;
    }
}

//****************************************************************
// This function is called when the mouse is moved over the scroolbar
// or when the mouse is moved when its down over the scrollbar

function onDragScrollbarLB()
{
    if (event.button != 1)
    {
        this.fDragging = false;
    }
    else
    {
        var nTop = event.y - this.nYOffset;    
        this.adjustScrollbarPositionLB( nTop );
    }
}

//****************************************************************
// This function is called to adjust the position of the selected
// item based upon the height / position of the listbox

function onKeyDownLB()
{
    var nIndex = this.nSelectedIndex;

    switch (event.keyCode)
    {
    case 36:        // home
        nIndex = 0;
        break;
    
    case 35:        // end
        nIndex = this.nCount - 1;
        break;
        
    case 38:        // up arrow
        nIndex--;
        break;

    case 40:        // down arrow
        nIndex++;
        break;
        
    case 33:        // page up
        nIndex -= Math.round(this.lb_sv.height / this.nPixelsPerLine);
        break;

    case 34:        // page dn
        nIndex += Math.round(this.lb_sv.height / this.nPixelsPerLine);
        break;
    }

    if (nIndex < 0)
    {
        nIndex = 0;
    }            
    else if (nIndex >= this.nCount)
    {
        nIndex = this.nCount - 1;
    }
    
    if (nIndex != this.nSelectedIndex)
    {
        this.setSelectedIndexLB( nIndex );
        this.adjustScrollbarForSelectionLB();
    }
}

//****************************************************************
// These functions are used for handling the up scroll on items

function onScrollUpMouseDownLB()
{
    if (event.button == 1)
    {
        this.fScrollingUp       = true;
        this.fScrollingUpHover  = true;
        this.onScrollUpEndMoveLB();
    }
}

function onScrollUpEndMoveLB()
{
    if (this.fScrollingUp && this.fScrollingUpHover)
    {
        this.scrollListboxLB( this.nPixelsPerLine );
        this.scrollup_button.moveTo( this.scrollup_button.left, this.scrollup_button.top, 100 );
    }
}

function onScrollUpMouseMoveLB()
{
    if (event.button != 1)
    {
        this.fScrollingUp = false;
        return;
    }

    var fOver = (   event.offsetX >= 0 && event.offsetX < 16 &&
                    event.offsetY >= 0 && event.offsetY < 16        );
    if (fOver != this.fScrollingUpHover)
    {
        this.fScrollingUpHover = fOver;
        this.onScrollUpEndMoveLB();
    }
}

function onScrollUpMouseUpLB()
{
    this.fScrollingUp = false;
}

//****************************************************************
// These functions are used for handling the down scroll on items

function onScrollDownMouseDownLB()
{
    if (event.button == 1)
    {
        this.fScrollingDown         = true;
        this.fScrollingDownHover    = true;
        this.onScrollDownEndMoveLB();
    }
}

function onScrollDownEndMoveLB()
{
    if (this.fScrollingDown && this.fScrollingDownHover)
    {
        this.scrollListboxLB( -this.nPixelsPerLine );
        this.scrolldown_button.moveTo( this.scrolldown_button.left, this.scrolldown_button.top, 100 );
    }
}

function onScrollDownMouseMoveLB()
{
    if (event.button != 1)
    {
        this.fScrollingDown = false;
        return;
    }

    var fOver = (   event.offsetX >= 0 && event.offsetX < 16 &&
                    event.offsetY >= 0 && event.offsetY < 16        );
    if (fOver != this.fScrollingDownHover)
    {
        this.fScrollingDownHover = fOver;
        this.onScrollDownEndMoveLB();
    }
}

function onScrollDownMouseUpLB()
{
    this.fScrollingDown = false;
}



//****************************************************************
// These functions are used for handling the scroll on the backing
// of the scrollbar

function onScrollBackMouseDownLB()
{
    if (event.button == 1)
    {
        this.fScrollingBack         = true;
        this.fScrollingBackHover    = true;
        this.fScrollingBackDir      = (event.y < this.scroll_button.top) ? 1 : -1;
        this.onScrollBackEndMoveLB();
    }
}

function onScrollBackEndMoveLB()
{
    if (this.fScrollingBack && this.fScrollingBackHover)
    {
        this.scrollListboxLB( this.fScrollingBackDir * this.lb_sv.height );
        this.scrollbacking_button.moveTo( this.scrollbacking_button.left, this.scrollbacking_button.top, 100 );
    }
}

function onScrollBackMouseMoveLB()
{
    if (event.button != 1)
    {
        this.fScrollingBack = false;
        return;
    }

    var fOver = (   event.offsetX >= 0 && event.offsetX < 16 &&
                    event.offsetY >= 0 && event.offsetY < (this.lb_sv.height - 32) );
    if (fOver != this.fScrollingBackHover)
    {
        this.fScrollingBackHover = fOver;
        this.onScrollBackEndMoveLB();
    }

    if (event.y < this.scroll_button.top)
    {
        this.fScrollingBackDir = 1;
    }
    else if (event.y >= (this.scroll_button.top + this.scroll_button.height))
    {
        this.fScrollingBackDir = -1;    
    }    
    else
    {
        this.fScrollingBackDir = 0;
    }
}

function onScrollBackMouseUpLB()
{
    this.fScrollingBack = false;
}




//****************************************************************
// Internal Functions
//****************************************************************



//****************************************************************
// This function is called to change the selected index

function setSelectedIndexLB( nIndex )
{
    this.nSelectedIndex = nIndex;
    if (this.onSelectionChange)
    {
        this.onSelectionChange();
    }
}


//****************************************************************
// This function returns a pixel position for the scrollbar that
// ensures that the current selection is visible

function adjustScrollbarForSelectionLB()
{
    var nTop    = this.items_text.top + (this.nSelectedIndex * this.nPixelsPerLine);

    if (nTop < 0)
    {
        this.scrollListboxLB( -nTop );
    }
    else if (nTop > (this.lb_sv.height - this.nPixelsPerLine))
    {
        this.scrollListboxLB( - (nTop - (this.lb_sv.height - this.nPixelsPerLine)) );
    }
    else
    {
        this.adjustSelectedItemLB();
    }
}


//****************************************************************
// This function adjusts the scrolling position of the text control
// and the scrollbar by the given number of pixels

function scrollListboxLB( nAmount )
{
    var nTop = this.items_text.top + nAmount;
    
    if (nTop < -(this.items_text.height - this.lb_sv.height) )
    {
        nTop = -(this.items_text.height - this.lb_sv.height);
    }
    else if (nTop > 0)
    {
        nTop = 0;    
    }
    
    // OK, now nTop should be the position we want this.items_text.top
    // to be... now we need to return the correct position of the
    // scrollbar to make this happen.

    // Now figure out the percentage of the text control that's visible

    var flTextPercent   = 0.0;
    if (this.items_text.height != 0)
    {
        flTextPercent   = (this.lb_sv.height / (this.items_text.height * 1.0));
    }
    if (flTextPercent > 1.0)
    {
        flTextPercent = 1.0;
    }

    // Now figure out the new position of our scrollbar

    var nScrollSize     = (1.0 - flTextPercent) * this.items_text.height;
    var flPercent       = (-nTop) / (nScrollSize * 1.0);

    var newPos          = Math.round((this.lb_sv.height - 32 - this.scroll_button.height) * flPercent) + 16;

    this.items_text.top     = nTop;

    if (newPos < 16)
    {
        newPos = 16;
    }
    else if (newPos > this.lb_sv.height - 16 - this.scroll_button.height)
    {
        newPos = this.lb_sv.height - 16 - this.scroll_button.height;
    }
    this.scroll_button.top = newPos;
    
    this.adjustSelectedItemLB();
}


//****************************************************************
// This function is called to adjust the position of the selected
// item based upon the height / position of the listbox

function adjustSelectedItemLB()
{
    var nTop    = this.items_text.top + (this.nSelectedIndex * this.nPixelsPerLine);
    var nOffset = 0;
    var nHeight = this.nPixelsPerLine;

    if (nTop < 0)
    {
        nOffset = -nTop;
        nTop    = 0;
        nHeight = this.nPixelsPerLine - nOffset;
    }
    else if (nTop > (this.lb_sv.height - this.nPixelsPerLine))
    {
        nHeight = this.lb_sv.height - nTop;
    }

    if (nHeight < 0)
    {
        nHeight = 0;
    }

    this.selected_sv.top        = (nTop > this.lb_sv.height) ? this.lb_sv.height : nTop;
    this.selected_sv.height     = nHeight;
    this.focus_button.height    = nHeight; 
    this.selected_text.top      = - ((this.nSelectedIndex * this.nPixelsPerLine) + nOffset);
}

//****************************************************************
// This function is called to adjust the position of the scrollbar
// based upon the position of the text in the listbox.

function adjustScrollbarPositionLB( newPos )
{
    // First determine if we should really have a scrollbar

    var fVisible = (this.lb_sv.height < this.items_text.height);

    this.scroll_button.visible          = fVisible;
    this.scrollup_button.enabled        = fVisible;
    this.scrolldown_button.enabled      = fVisible;
    this.scrollbacking_button.enabled   = fVisible;

    // And now determine where the new position should be

    if (newPos < 16)
    {
        newPos = 16;
    }
    else if (newPos > this.lb_sv.height - 16 - this.scroll_button.height)
    {
        newPos = this.lb_sv.height - 16 - this.scroll_button.height;
    }

    this.scroll_button.top = newPos;
    
    // And once we've changed the position, then scroll the text
    // to the appopriate position for the scrollbar    
    
    this.scrollTextLB();
}

//****************************************************************
// This function moves the text in accordance with the position of
// the slider bar

function scrollTextLB()
{
    // First figure out the scrollbar position

    var nMax        = this.lb_sv.height - 32 - this.scroll_button.height;
    var nVal        = this.scroll_button.top - 16;

    var flPercent   = 0.0;
    if (nMax != 0)
    {
        flPercent   = nVal / (nMax * 1.0);
    }

    // Now figure out the percentage of the text control that's visible

    var flTextPercent   = 0.0;
    if (this.items_text.height != 0)
    {
        flTextPercent   = (this.lb_sv.height / (this.items_text.height * 1.0));
    }
    if (flTextPercent > 1.0)
    {
        flTextPercent = 1.0;
    }

    // Now figure out our new position

    var nScrollSize     = (1.0 - flTextPercent) * this.items_text.height;
    var nScrollPos      = nScrollSize * flPercent;

    this.items_text.top = -nScrollPos;
    
    // And when we change the position of the text, then we must adjust
    // the selected item
    
    this.adjustSelectedItemLB();
}



